@@ -9,7 +9,6 @@
     \inmodule QtGui
     \reentrant
     \ingroup painting
-    \ingroup io
 
     QImageWriter supports setting format specific options, such as
     compression level and quality, prior to storing the
@@ -315,9 +314,9 @@ QImageWriter::QImageWriter(const QString &fileName, const QByteArray &format)
 */
 QImageWriter::~QImageWriter()
 {
+    delete d->handler;
     if (d->deleteDevice)
         delete d->device;
-    delete d->handler;
     delete d;
 }
 
@@ -362,13 +361,13 @@ QByteArray QImageWriter::format() const
 */
 void QImageWriter::setDevice(QIODevice *device)
 {
+    delete d->handler;
+    d->handler = nullptr;
     if (d->device && d->deleteDevice)
         delete d->device;
 
     d->device = device;
     d->deleteDevice = false;
-    delete d->handler;
-    d->handler = nullptr;
 }
 
 /*!
@@ -589,8 +588,6 @@ QImageIOHandler::Transformations QImageWriter::transformation() const
 }
 
 /*!
-    \since 4.1
-
     Sets the image text associated with the key \a key to
     \a text. This is useful for storing copyright information
     or other information about the image. Example:
@@ -711,8 +708,6 @@ QString QImageWriter::errorString() const
 }
 
 /*!
-    \since 4.2
-
     Returns \c true if the writer supports \a option; otherwise returns
     false.
 
