@@ -159,8 +159,6 @@ static void qt_debug_path(const QPainterPath &path)
     toReversed() function. There are also several functions to convert
     this painter path object into a polygon representation.
 
-    \tableofcontents
-
     \section1 Composing a QPainterPath
 
     A QPainterPath object can be constructed as an empty path, with a
@@ -517,10 +515,8 @@ QPainterPath::QPainterPath(const QPainterPath &other) = default;
 */
 
 QPainterPath::QPainterPath(const QPointF &startPoint)
-    : d_ptr(new QPainterPathPrivate)
+    : d_ptr(new QPainterPathPrivate(startPoint))
 {
-    Element e = { startPoint.x(), startPoint.y(), MoveToElement };
-    d_func()->elements << e;
 }
 
 void QPainterPath::detach()
@@ -1135,6 +1131,9 @@ void QPainterPath::addEllipse(const QRectF &boundingRect)
     that the left end of the text's baseline lies at the specified \a
     point.
 
+    Some fonts may yield overlapping subpaths and will require the
+    \c Qt::WindingFill fill rule for correct rendering.
+
     \table 100%
     \row
     \li \inlineimage qpainterpath-addtext.png
@@ -1143,7 +1142,7 @@ void QPainterPath::addEllipse(const QRectF &boundingRect)
     \endtable
 
     \sa QPainter::drawText(), {QPainterPath#Composing a
-    QPainterPath}{Composing a QPainterPath}
+    QPainterPath}{Composing a QPainterPath}, setFillRule()
 */
 void QPainterPath::addText(const QPointF &point, const QFont &f, const QString &text)
 {
@@ -1488,7 +1487,7 @@ QRectF QPainterPath::controlPointRect() const
 
 bool QPainterPath::isEmpty() const
 {
-    return !d_ptr || (d_ptr->elements.size() == 1 && d_ptr->elements.first().type == MoveToElement);
+    return !d_ptr || (d_ptr->elements.size() == 1 && d_ptr->elements.constFirst().type == MoveToElement);
 }
 
 /*!
@@ -2441,14 +2440,14 @@ QDataStream &operator>>(QDataStream &s, QPainterPath &p)
     int size;
     s >> size;
 
-    if (size == 0)
+    if (size == 0) {
+        p = {};
         return s;
+    }
 
     p.ensureData(); // in case if p.d_func() == 0
-    if (p.d_func()->elements.size() == 1) {
-        Q_ASSERT(p.d_func()->elements.at(0).type == QPainterPath::MoveToElement);
-        p.d_func()->elements.clear();
-    }
+    p.detach();
+    p.d_func()->elements.clear();
     for (int i=0; i<size; ++i) {
         int type;
         double x, y;
@@ -2471,9 +2470,7 @@ QDataStream &operator>>(QDataStream &s, QPainterPath &p)
     s >> fillRule;
     Q_ASSERT(fillRule == Qt::OddEvenFill || fillRule == Qt::WindingFill);
     p.d_func()->fillRule = Qt::FillRule(fillRule);
-    p.d_func()->dirtyBounds = true;
-    p.d_func()->dirtyControlBounds = true;
-    if (errorDetected)
+    if (errorDetected || p.d_func()->elements.isEmpty())
         p = QPainterPath();  // Better than to return path with possibly corrupt datastructure, which would likely cause crash
     return s;
 }
@@ -3365,11 +3362,11 @@ void QPainterPath::computeControlPointRect() const
 #ifndef QT_NO_DEBUG_STREAM
 QDebug operator<<(QDebug s, const QPainterPath &p)
 {
+    QDebugStateSaver saver(s);
     s.nospace() << "QPainterPath: Element count=" << p.elementCount() << Qt::endl;
     const char *types[] = {"MoveTo", "LineTo", "CurveTo", "CurveToData"};
     for (int i=0; i<p.elementCount(); ++i) {
         s.nospace() << " -> " << types[p.elementAt(i).type] << "(x=" << p.elementAt(i).x << ", y=" << p.elementAt(i).y << ')' << Qt::endl;
-
     }
     return s;
 }
