@@ -1,7 +1,7 @@
 // Copyright (C) 2019 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
-#include "private/qnativesocketengine_p.h"
+#include "private/qnativesocketengine_p_p.h"
 #include "private/qnetconmonitor_p.h"
 
 #include "private/qobject_p.h"
@@ -229,20 +229,20 @@ bool QNetworkConnectionMonitor::startMonitoring()
 
     auto queue = qt_reachability_queue();
     if (!queue) {
-        qWarning(lcNetMon, "Failed to create a dispatch queue to schedule a probe on");
+        qCWarning(lcNetMon, "Failed to create a dispatch queue to schedule a probe on");
         return false;
     }
 
     SCNetworkReachabilityContext context = {};
     context.info = d;
     if (!SCNetworkReachabilitySetCallback(d->probe, QNetworkConnectionMonitorPrivate::probeCallback, &context)) {
-        qWarning(lcNetMon, "Failed to set a reachability callback");
+        qCWarning(lcNetMon, "Failed to set a reachability callback");
         return false;
     }
 
 
     if (!SCNetworkReachabilitySetDispatchQueue(d->probe, queue)) {
-        qWarning(lcNetMon, "Failed to schedule a reachability callback on a queue");
+        qCWarning(lcNetMon, "Failed to schedule a reachability callback on a queue");
         return false;
     }
 
